@@ -538,7 +538,6 @@ initialise_error(E) :-
     fail.
 
 initialise_prolog :-
-    '$clean_history',
     apply_defines,
     init_optimise,
     '$run_initialization',
@@ -581,7 +580,6 @@ user_thread_init :-
     argv_prolog_files(Files, ScriptMode),
     load_init_file(ScriptMode),                 % -f file
     catch(setup_colors, E, print_message(warning, E)),
-    '$load_history',
     win_associated_files(Files),                % swipl-win: cd and update title
     '$load_script_file',                        % -s file (may be repeated)
     load_associated_files(Files),
@@ -602,6 +600,9 @@ user_thread_init :-
 
 %!  setup_theme
 
+:- multifile
+    prolog:theme/1.
+
 setup_theme :-
     current_prolog_flag(theme, Theme),
     exists_source(library(theme/Theme)),
@@ -750,8 +751,7 @@ run_init_goal(Goal, Ctx) :-
     (   catch_with_backtrace(user:Goal, E, true)
     ->  (   var(E)
         ->  true
-        ;   print_message(error, init_goal_failed(E, Ctx)),
-            halt(2)
+        ;   init_goal_failed(E, Ctx)
         )
     ;   (   current_prolog_flag(verbose, silent)
         ->  Level = silent
@@ -761,6 +761,16 @@ run_init_goal(Goal, Ctx) :-
         halt(1)
     ).
 
+init_goal_failed(E, Ctx) :-
+    print_message(error, init_goal_failed(E, Ctx)),
+    init_goal_failed(E).
+
+init_goal_failed(_) :-
+    thread_self(main),
+    !,
+    halt(2).
+init_goal_failed(_).
+
 %!  init_debug_flags is det.
 %
 %   Initialize the various Prolog flags that   control  the debugger and
@@ -818,18 +828,14 @@ setup_history :-
         load_setup_file(library(prolog_history))
     ->  prolog_history(enable)
     ;   true
-    ),
-    set_default_history,
-    '$load_history'.
+    ).
 
 %!  setup_readline
 %
 %   Setup line editing.
 
 setup_readline :-
-    (   current_prolog_flag(readline, swipl_win)
-    ->  true
-    ;   stream_property(user_input, tty(true)),
+    (   stream_property(user_input, tty(true)),
         current_prolog_flag(tty_control, true),
         \+ getenv('TERM', dumb),
         (   current_prolog_flag(readline, ReadLine)
@@ -837,7 +843,13 @@ setup_readline :-
         ;   ReadLine = true
         ),
         readline_library(ReadLine, Library),
-        load_setup_file(library(Library))
+        (   load_setup_file(library(Library))
+        ->  true
+        ;   print_message(warning,
+                          error(existence_error(library, library(Library)),
+                                _)),
+            fail
+        )
     ->  set_prolog_flag(readline, Library)
     ;   set_prolog_flag(readline, false)
     ).
@@ -851,7 +863,6 @@ readline_library(false, _) :-
 readline_library(Library, Library).
 
 preferred_readline(editline).
-preferred_readline(readline).
 
 %!  load_setup_file(+File) is semidet.
 %
@@ -861,7 +872,7 @@ load_setup_file(File) :-
     catch(load_files(File,
                      [ silent(true),
                        if(not_loaded)
-                     ]), _, fail).
+                     ]), error(_,_), fail).
 
 
 %!  setup_app is det.
@@ -1240,6 +1251,9 @@ read_expanded_query(BreakLev, ExpandedQuery, ExpandedBindings) :-
 %   !-based history is enabled. The second is   used  if we have command
 %   line editing.
 
+:- multifile
+    prolog:history/2.
+
 :- if(current_prolog_flag(emscripten, true)).
 read_query(_Prompt, Goal, Bindings) :-
     '$can_yield',
@@ -1248,14 +1262,13 @@ read_query(_Prompt, Goal, Bindings) :-
     term_string(Goal, GoalString, [variable_names(Bindings)]).
 :- endif.
 read_query(Prompt, Goal, Bindings) :-
-    current_prolog_flag(history, N),
-    integer(N), N > 0,
+    prolog:history(current_input, enabled),
     !,
     read_term_with_history(
         Goal,
         [ show(h),
           help('!h'),
-          no_save([trace, end_of_file]),
+          no_save([trace]),
           prompt(Prompt),
           variable_names(Bindings)
         ]).
@@ -1264,7 +1277,6 @@ read_query(Prompt, Goal, Bindings) :-
     repeat,                                 % over syntax errors
     prompt1(Prompt1),
     read_query_line(user_input, Line),
-    '$save_history_line'(Line),             % save raw line (edit syntax errors)
     '$current_typein_module'(TypeIn),
     catch(read_term_from_atom(Line, Goal,
                               [ variable_names(Bindings),
@@ -1273,20 +1285,30 @@ read_query(Prompt, Goal, Bindings) :-
           (   print_message(error, E),
               fail
           )),
-    !,
-    '$save_history_event'(Line).            % save event (no syntax errors)
+    !.
 
-%!  read_query_line(+Input, -Line) is det.
+%!  read_query_line(+Input, -Query:atom) is det.
+%
+%   Read a query as an atom. If Query is '$silent'(Goal), execute `Goal`
+%   in module `user` and read the   next  query. This supports injecting
+%   goals in some GNU-Emacs modes.
 
 read_query_line(Input, Line) :-
     stream_property(Input, error(true)),
     !,
     Line = end_of_file.
 read_query_line(Input, Line) :-
-    catch(read_term_as_atom(Input, Line), Error, true),
+    catch(read_term_as_atom(Input, Line0), Error, true),
     save_debug_after_read,
     (   var(Error)
-    ->  true
+    ->  (   catch(term_string(Goal, Line0), error(_,_), fail),
+            Goal = '$silent'(SilentGoal)
+        ->  Error = error(_,_),
+            catch_with_backtrace(ignore(SilentGoal), Error,
+                                 print_message(error, Error)),
+            read_query_line(Input, Line)
+        ;   Line = Line0
+        )
     ;   catch(print_message(error, Error), _, true),
         (   Error = error(syntax_error(_),_)
         ->  fail
@@ -1341,24 +1363,6 @@ delete_leading_blanks([' '|T0], T) :-
 delete_leading_blanks(L, L).
 
 
-%!  set_default_history
-%
-%   Enable !-based numbered command history. This  is enabled by default
-%   if we are not running under GNU-emacs  and   we  do not have our own
-%   line editing.
-
-set_default_history :-
-    current_prolog_flag(history, _),
-    !.
-set_default_history :-
-    (   (   \+ current_prolog_flag(readline, false)
-        ;   current_prolog_flag(emacs_inferior_process, true)
-        )
-    ->  create_prolog_flag(history, 0, [])
-    ;   create_prolog_flag(history, 25, [])
-    ).
-
-
                  /*******************************
                  *        TOPLEVEL DEBUG        *
                  *******************************/
